#include <QTime>
#include <QDebug>
#include <stdio.h>
#include <assert.h>
#include "config.h"
#include "audioQt.h"
#include "lib.h"

#define AUDIO_TIMEOUT		20 
#define AUDIO_BUF_SIZE		2048

Audio::Audio(char* dev, char* mixerDev, char* mixerVol, char* mixerMic, int rate)  {
	tx = false;
	mute = false;
	noutput_items = AUDIO_BUF_SIZE;

	QAudioFormat format;
	format.setSampleRate(rate);
	format.setChannelCount(1);
	format.setSampleSize(16);
	format.setCodec("audio/pcm");
	format.setByteOrder(QAudioFormat::LittleEndian);
	format.setSampleType(QAudioFormat::SignedInt);

	QList<QAudioDeviceInfo> audioDevicesOut =  QAudioDeviceInfo::availableDevices(QAudio::AudioOutput);
	QList<QAudioDeviceInfo> audioDevicesIn =  QAudioDeviceInfo::availableDevices(QAudio::AudioInput);
	
	QAudioDeviceInfo infoIn  = audioDevicesIn.at(0);
	QAudioDeviceInfo infoOut = audioDevicesOut.at(0);

	if (!infoIn.isFormatSupported(format)) {
		qWarning()<<"AudioInput: default format not supported try to use nearest";
		format = infoIn.nearestFormat(format);
	}
	if (!infoOut.isFormatSupported(format)) {
		qWarning()<<"AudioInput: default format not supported try to use nearest";
		format = infoOut.nearestFormat(format);
	}
	audioOutput = new QAudioOutput(infoOut, format);
	audioOutput->setBufferSize(1500);
	audioInput = new QAudioInput(infoIn, format);
	audioInDev = NULL;
	audioOutDev = NULL;
	audioOutDev = audioOutput->start();
	
	mutex = new QMutex();
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
	timer->start(AUDIO_TIMEOUT);
	minOutSize=1000;
    audioRun = true;
}

Audio::~Audio() {
	audioOutput->stop();
	delete audioOutput;
	delete audioInput;
}

void Audio::audioMute(bool m) {
	mute = m;
}

void Audio::setTX(bool m) {
	if (m)
		audioInDev = audioInput->start();
	else {
		audioInput->stop();
		audioInDev = NULL;
	}
}

void Audio::audioRX(QByteArray out) {
	if (!audioRun)
		return;
	audioOutBuf.append(out);
	//qDebug() << "free:" << audioOutput->bytesFree();
	if (audioOutput->bytesFree() && audioOutBuf.size()>minOutSize) {
		int len = audioOutDev->write(audioOutBuf);
		audioOutBuf.remove(0, len);
		//qDebug() << "Write:" << len << "buf:" << audioOutBuf.size();
		minOutSize=0;
		if (audioOutBuf.size()>AUDIO_BUF_SIZE) 
			audioOutBuf.remove(0,AUDIO_BUF_SIZE);
	}

	if (audioInDev) {
		QByteArray audioInData =  audioInDev->readAll();
		//qDebug() << "Read:" << audioInData.size();
		if (audioInData.size())
			emit audioTX(QByteArray(audioInData));
	}
}

void Audio::timeout() {
}

void Audio::terminate() {
    audioRun = false;
}

void Audio::run() {
    while(audioRun) {
		msleep(200);
	}
}

#ifdef USE_MIXER
void Audio::setVolume(int volume) {
	snd_mixer_selem_set_playback_volume_all(elemVol, volume * volumeMax / 256);
}

void Audio::setMic(int mic) {
	snd_mixer_selem_set_playback_volume_all(elemMic, mic * micMax / 256);
}
#endif
