#include <QTime>
#include <stdio.h>
#include "control.h"
#include "config.h"
#include "lib.h"
#include "display_touch.h"
#include "display_lcd.h"

#define X(a, b) b,
const char *CmdString[] = { CMDS };
#undef X
extern bool touchscreen;
extern bool defaults;

const char *SrcString[] = {"Display","Sdr", "Control", "Key", "DisplayKey"};


Control::Control(char *ip, char* audiodev, char* audiodevMixer, char* mixerVolume, char* mixerMic, int sampleRate)  {

	qDebug() << "RaSDR Version " << VERSION;
	initReady = false;

	txChanged = 0;
	settings =  new QSettings("sdr","rasdr");
	readGlobalSettings(settings);
	if (ip)
		hiqsdrIP = ip;
	else
		hiqsdrIP    = "192.168.2.196";

	for (int i=0;i<CMD_LAST;i++)
		config[i] = 0;
	printf("rasdr SVN: %s\n",SVNREV);

	config[CMD_AGC_DEC] = 10000;
	config[CMD_ANT] = 0;
	config[CMD_DISP_MODE] = 1;
	config[CMD_DIV1] = CIC_A_DIV;
	config[CMD_DIV2] = CIC_B_DIV;
	config[CMD_EXIT] = 1;
	config[CMD_FFT_COMPLEX] = 1;
	config[CMD_FFT_SAMPLE_RATE] = RX_CLOCK/CIC_A_DIV;
	config[CMD_FFT_SIZE] = FFT_SIZE;
	config[CMD_FFT_TIME] = 200;
	config[CMD_RX_FILTER_GAIN] = 3;
	config[CMD_TX_FILTER_GAIN] = 3;
	config[CMD_FILTER_RX_CUT] = 120;
	config[CMD_FILTER_RX_HI] = 2700;
	config[CMD_FILTER_RX_LO] = 240;
	config[CMD_FILTER_TX_CUT] = 250;
	config[CMD_FILTER_TX_HI] = 2700;
	config[CMD_FILTER_TX_LO] = 200;
	config[CMD_GAIN] = 0;
	config[CMD_INTERNAL] =  C_INTERNAL_INT;
	config[CMD_MODE] = MODE_LSB;
	config[CMD_POWER] = 0;
	config[CMD_PREAMP] = 0;
	config[CMD_PRESEL] = 1;
	config[CMD_RAW_SRC] = RAW_SRC_CIC_B;
	config[CMD_RX_FREQ] = 7100000;
	config[CMD_SAMPLE_RATE] = DEFAULT_SAMPLE_RATE;
	config[CMD_STEP] = 100;
	config[CMD_SWR] = 0;
	config[CMD_TX] = 0;
	config[CMD_TX_FREQ] = config[CMD_RX_FREQ];
	config[CMD_TX_POWER] = 100;
	config[CMD_VOLUME] = 30;
	config[CMD_MIC] = 0;
	config[CMD_WATERFALL_MAX] = WATERFALL_MAX;
	config[CMD_WATERFALL_MIN] = WATERFALL_MIN;
	config[CMD_TX_DELAY] = 0;
	config[CMD_FULL_DUPLEX] = 0;
	config[CMD_NB_LEVEL] = 0xff;
	config[CMD_TX_RX] = 1;
	config[CMD_SELECT0] = 1;
	config[CMD_SELECT1] = 1;
	config[CMD_DISP_COLOR] = 1;

	if (!defaults)
		readSettings();

	config[CMD_SAMPLE_RATE] = sampleRate;
	config[CMD_TX_FREQ] = config[CMD_RX_FREQ];

	if (touchscreen)
		qApp->setOverrideCursor(QCursor( Qt::BlankCursor ));
	disp = new Display_touch();
	//disp->start();

	sdr = new Hiqsdr(hiqsdrIP, config[CMD_SAMPLE_RATE]);
	sdr->setSampleRate(config[CMD_SAMPLE_RATE]);
	sdr->setFFT(config[CMD_FFT_TIME], config[CMD_FFT_SIZE], RAW_SRC_FULL);
	sdr->start(QThread::HighPriority);

	audio = new Audio(audiodev, audiodevMixer, mixerVolume, mixerMic, config[CMD_SAMPLE_RATE]);
	connect(audio, SIGNAL(audioTX(QByteArray)), sdr, SLOT(writeHiqsdr(QByteArray)), Qt::QueuedConnection);

	fir = new firdes();
	timer = new QTimer(this);

#ifdef UNIX
	//keyReader = new KeyReader();

	//connect(keyReader, SIGNAL(controlCommand(int,int,int)), this, SLOT(controlCommand(int,int,int))); 
#endif
	connect (sdr, SIGNAL(audioRX(QByteArray)), audio, SLOT(audioRX(QByteArray)));
	connect (sdr, SIGNAL(audioRXRaw(QByteArray)), this, SLOT(audioRXRaw(QByteArray)));
	connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
	connect(disp, SIGNAL(command(int, int, int)), this, SLOT(controlCommand(int, int, int)));
	connect(this, SIGNAL(displaySet(int, int, int)), disp, SLOT(displaySet(int, int, int)));
	connect(sdr, SIGNAL(fftData(QByteArray)), disp, SLOT(fftData(QByteArray)), Qt::QueuedConnection);
	connect(sdr, SIGNAL(controlCommand(int,int,int)), this, SLOT(controlCommand(int,int,int)));

#ifdef UNIX
	//keyReader->start();
#endif
	hamLibSocket = new QUdpSocket(this);
	hamLibSocket->bind(13222, QUdpSocket::ShareAddress);
	connect(hamLibSocket, SIGNAL(readyRead()), this, SLOT(readHamLibUDPData()));

	hpsdrSocket = new QUdpSocket(this);
	hpsdrSocket->bind(1024, QUdpSocket::ShareAddress);
	connect(hpsdrSocket, SIGNAL(readyRead()), this, SLOT(readHpsdrUDPData()));


	audio->start(QThread::TimeCriticalPriority);

	hpRxBufPos=0;
	sender=0;
	senderPort=0;
	cCnt=0;

	hp_receivers=1;
	memset(hpRxBuf, 0, sizeof(hpRxBuf));

	for (int i=0;i<CMD_INIT_END;i++) 
		controlCommand(SRC_CTL, i, config[i], true);
	initReady = true;
}

Control::~Control() {
	writeSettings();
	delete timer;
	delete fir;
	audio->terminate();
	audio->wait();
	delete audio;
	sdr->terminate();
	sdr->wait();
	delete sdr;
	delete disp;
#ifdef UNIX
	//keyReader->restore();
	//keyReader->terminate();
	//keyReader->wait();
	//delete keyReader;
#endif
	delete settings;
}

void Control::timeout() {
	delete this;
}

void Control::restoreFilter() {
#if 1
	sdr->setFilter(fir->complex_band_pass_taps(config[CMD_RX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*8, 
				-(config[CMD_SAMPLE_RATE]/2-config[CMD_FILTER_RX_CUT]), config[CMD_SAMPLE_RATE]/2-config[CMD_FILTER_RX_CUT], 1700), FILTER_RX);
	sdr->setFilter(fir->complex_band_pass_taps(config[CMD_TX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*4, 
				-(config[CMD_SAMPLE_RATE]/2-config[CMD_FILTER_TX_CUT]), config[CMD_SAMPLE_RATE]/2-config[CMD_FILTER_TX_CUT], 420), FILTER_TX);
#else
	std::vector<gr_complex> v_coef;
	for (int i=0;i<arraysize(coef);i++)
		v_coef.push_back(coef[i]);
	sdr->setFilter(v_coef, FILTER_RX);

	v_coef.clear();
	for (int i=0;i<arraysize(tx_coef);i++)
		v_coef.push_back(tx_coef[i]);
	sdr->setFilter(v_coef, FILTER_TX);
#endif
}

void Control::controlCommand(int src, int cmd, int par, bool initial) {
	PDEBUG(MSG1, "%s -> Control: %s, Par: %i",SrcString[src], CmdString[cmd],par);
	if (!initial && (config[cmd] == par) && (cmd < CMD_ENCODER0))
		return;
	config[cmd]=par;

	if (src != SRC_DISP)
		emit displaySet(SRC_CTL, cmd, par);

	switch ((CtlCmd)cmd) {
		case CMD_EXIT:	// CTL_C
			qDebug() << "exit, restore filter";
			sdr->setPtt(false);	
			sdr->setNBLevel(255);	
			restoreFilter();
			timer->start(1600);
			break;
		case CMD_RX_FREQ:
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			break;
		case CMD_TX_FREQ:
			sdr->setTXFreq(config[CMD_TX_FREQ]);	
			break;
		case CMD_MODE:
			setMode();
			break;
		case CMD_RAW_SRC:
			sdr->setFFT(config[CMD_FFT_TIME], config[CMD_FFT_SIZE], config[CMD_RAW_SRC]);
			// no break;
		case CMD_DIV1:
		case CMD_DIV2:
			if (config[CMD_RAW_SRC] == RAW_SRC_CIC_A) 
				emit displaySet(SRC_CTL, CMD_FFT_SAMPLE_RATE, RX_CLOCK/config[CMD_DIV1]);
			else
				emit displaySet(SRC_CTL, CMD_FFT_SAMPLE_RATE, RX_CLOCK/config[CMD_DIV1]/config[CMD_DIV2]);
			break;
		case CMD_FILTER_RX_LO:
			setMode();
			break;
		case CMD_FILTER_RX_HI:
			setMode();
			break;
		case CMD_TX:
			txChanged = 50;
			if (src != SRC_SDR)
				sdr->setPtt(config[CMD_TX]);	
			audio->setTX(config[CMD_TX]);
			break;
		case CMD_VOLUME:
#ifdef USE_MIXER
			audio->setVolume(checkRange(config[CMD_VOLUME],0,255));
#else
			sdr->setVolume((double)config[CMD_VOLUME]/256);
#endif
			break;
		case CMD_MIC:
#ifdef USE_MIXER
			audio->setMic(checkRange(config[CMD_MIC],0,255));
#else
			sdr->setMicGain((double)config[CMD_MIC]/25);
#endif
			break;
		case CMD_PREAMP:
			sdr->setAttenuator(-config[CMD_PREAMP]);	
			break;
		case CMD_ANT:
			sdr->setAnt(config[CMD_ANT]);	
			break;
		case CMD_AGC_DEC:
			sdr->setAGCDec(1.0/config[CMD_AGC_DEC]);
			break;
		case CMD_TX_POWER:
			sdr->setTXLevel(config[CMD_TX_POWER]);
			break;
		case CMD_GAIN:
			if (config[CMD_GAIN]==0)
				sdr->setGain(0);
			else
				sdr->setGain(exp(config[CMD_GAIN]/5));
			break;
		case CMD_FREQ_STEP_UP:
			config[CMD_RX_FREQ] += config[CMD_STEP];
			if (config[CMD_RX_FREQ] > RX_CLOCK/2)
				config[CMD_RX_FREQ] = RX_CLOCK/2;
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			break;
		case CMD_FREQ_STEP_DOWN:
			config[CMD_RX_FREQ] -= config[CMD_STEP];
			if (config[CMD_RX_FREQ] < 0)
				config[CMD_RX_FREQ] = 0;
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			break;
		case CMD_ENCODER0:
			break;
		case CMD_INTERNAL:
			sdr->setTXRate(config[CMD_SAMPLE_RATE]);
			sdr->setSampleRate(config[CMD_SAMPLE_RATE]);
			config[CMD_SAMPLE_RATE]=config[CMD_SAMPLE_RATE];
			switch (config[CMD_INTERNAL]) {
				case C_INTERNAL_INT:
					// restore state
					sdr->setActive(true);
					sdr->setAudioOff(false);
					sdr->startRX();
					setMode();
					break;
				case C_INTERNAL_OFF:
					sdr->setActive(false);
					sdr->setAudioOff(true);
					sdr->stopRX();
					restoreFilter();
					break;
				case C_INTERNAL_EXT:
					sdr->setActive(true);
					sdr->setAudioOff(true);
					sdr->stopRX();
					restoreFilter();
					break;
				case C_INTERNAL_HPSDR:
					sdr->setTXRate(48000);
					sdr->setActive(true);
					sdr->setAudioOff(true);
					sdr->startRX();
					restoreFilter();
					break;
			}
			break;
		case CMD_TWO_TONE_TEST:
			{
				double l1,l2;
				double f1 = getSettings(settings,"Common/testToneFreq1",800);
				double f2 = getSettings(settings,"Common/testToneFreq2",1200);
				double f3 = getSettings(settings,"Common/testToneFreqSingle",1000);
				switch (config[CMD_TWO_TONE_TEST]) {
					case 0:
						l1=0;l2=0;
						break;
					case 1:
						l1=1.0/2/sqrt(10000);l2=l1;
						break;
					case 2:
						l1=1.0/2/sqrt(1000);l2=l1;
						break;
					case 3:
						l1=1.0/2/sqrt(100);l2=l1;
						break;
					case 4:
						l1=1.0/2/sqrt(10);l2=l1;
						break;
					case 5:
						l1=1.0/2;l2=l1;
						break;
					default:
						f1=f3;l1=1;f2=0;l2=0;
				}
				sdr->setToneTest(config[CMD_TWO_TONE_TEST]>0, f1,l1,f2,l2);
			}
			break;
		case CMD_TX_DELAY:
			sdr->setTxDelay(config[CMD_TX_DELAY]);
			break;
		case CMD_FULL_DUPLEX:
			sdr->setFdx(config[CMD_FULL_DUPLEX]);
			break;
		case CMD_NB_LEVEL:
			sdr->setNBLevel(config[CMD_NB_LEVEL]);
			break;
		case CMD_SAMPLE_RATE:
			sdr->setSampleRate(config[CMD_SAMPLE_RATE]);
			break;
		case CMD_NOTCH:
			sdr->setNotch(config[CMD_NOTCH]);
			break;
		case CMD_SQUELCH:
			sdr->setSquelch(config[CMD_SQUELCH]);
			break;
		case CMD_FFT_SIZE:
			sdr->setFFT(config[CMD_FFT_TIME], config[CMD_FFT_SIZE], config[CMD_RAW_SRC]);
			break;
		case CMD_LAYOUT_CHANGED:
			if (initReady) 
				for (int i=0;i<CMD_INIT_END;i++) 
					controlCommand(SRC_CTL, i, config[i], true);
			break;
		case CMD_IN_LEVEL:
		case CMD_STEP:
		case CMD_RSSI:
		case CMD_POWER:
		case CMD_SWR:
		case CMD_RECONNECT:
		case CMD_DISP_MODE:
		case CMD_BAND:
		case CMD_ADC_OV:
		case CMD_TX_RX:
		case CMD_DISP_COLOR:
		case CMD_ENCODER1 ... CMD_ENCODER7:
			break;
		default:
			PDEBUG(ERR1, "Control: unhandled command: %s, par %i",CmdString[cmd],par);
	}
}

void Control::setMode() {
	sdr->setMode(config[CMD_MODE]);
	switch(config[CMD_MODE]) {
		case MODE_LSB:
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_RX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*8, -config[CMD_FILTER_RX_HI], -config[CMD_FILTER_RX_LO], RX_TAPS), FILTER_RX);
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_TX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*4, -config[CMD_FILTER_RX_HI], -config[CMD_FILTER_RX_LO], TX_TAPS), FILTER_TX);
			break;
		case MODE_USB:
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_RX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*8, config[CMD_FILTER_RX_LO], config[CMD_FILTER_RX_HI], RX_TAPS), FILTER_RX);
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_TX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*4, config[CMD_FILTER_RX_LO], config[CMD_FILTER_RX_HI], TX_TAPS), FILTER_TX);
			break;
		case MODE_CW:
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_RX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*8, -config[CMD_FILTER_RX_LO]/2, +config[CMD_FILTER_RX_LO]/2, RX_TAPS), FILTER_RX);
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_TX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*4, -config[CMD_FILTER_TX_CUT]/2, +config[CMD_FILTER_TX_CUT]/2, TX_TAPS), FILTER_TX);
			break;
		case MODE_AM:
		case MODE_FM:
			sdr->setRXFreq(config[CMD_RX_FREQ]);	
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_RX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*8, -config[CMD_FILTER_RX_HI], config[CMD_FILTER_RX_HI], RX_TAPS), FILTER_RX);
			sdr->setFilter(fir->complex_band_pass_taps(config[CMD_TX_FILTER_GAIN], config[CMD_SAMPLE_RATE]*4, -config[CMD_FILTER_RX_HI], config[CMD_FILTER_RX_HI], TX_TAPS), FILTER_TX);
			break;
	}
}

void Control::readSettings() {
	settings->beginGroup("Version");
	int version = settings->value("verint",0).toInt();
	settings->endGroup();

	if (version<VERSION_INT) {
		qDebug() << "old settings version is to old (" << version << "), skip";
		return;
	}

	settings->beginGroup("Control");
	settings->beginReadArray("Config");
	for (int i=0;i<CMD_INIT_END;i++) {
		settings->setArrayIndex(i);
		for(int j=0;j<CMD_INIT_END;j++) {
			if (CmdString[j] == settings->value("name").toString())
				config[j] = settings->value("val",config[j]).toInt();
		}
	}
	settings->endArray();
	settings->endGroup();
}

void Control::writeSettings() {
	settings->beginGroup("Version");
	settings->setValue("ver", VERSION);
	settings->setValue("verint", VERSION_INT);
	settings->endGroup();

	settings->beginGroup("Control");
	settings->beginWriteArray("Config");
	for (int i=0;i<CMD_INIT_END;i++) {
		settings->setArrayIndex(i);
		settings->setValue("name", CmdString[i]);
		settings->setValue("val", config[i]);
	}
	settings->endArray();
	settings->endGroup();
}

void Control::readHamLibUDPData() {
	while (hamLibSocket->hasPendingDatagrams()) {
		QByteArray datagram;
		datagram.resize(hamLibSocket->pendingDatagramSize());
		QHostAddress sender;
		quint16 senderPort;
		QRegExp rx("(\\S) *(\\S*)");

		hamLibSocket->readDatagram(datagram.data(), datagram.size(), &sender, &senderPort);
		if (rx.indexIn(datagram)!=-1) {

			QString tag = rx.cap(1);
			QString arg = rx.cap(2);
			qint64 iarg = arg.toLong();

			if (tag == "f") {
				controlCommand(SRC_CTL, CMD_RX_FREQ, iarg, false);
				controlCommand(SRC_CTL, CMD_TX_FREQ, iarg, false);
			}
			if (tag == "F") {
				datagram.clear();
				datagram.append(QString("f %1\n").arg(config[CMD_RX_FREQ],12));
				hamLibSocket->writeDatagram(datagram, sender, senderPort);
			}
			if (tag == "t") {
				controlCommand(SRC_CTL, CMD_TX, iarg>0, false);
			}
		}
	}
}

// HPSDR protocoll 
void Control::readHpsdrUDPData() {
	while (hpsdrSocket->hasPendingDatagrams()) {
		QByteArray data,datagram;
		datagram.resize(hamLibSocket->pendingDatagramSize());
		uint8_t buf[1524];
		uint8_t *c = buf+11;
		int rate[] = {48000,96000,192000,384000};
		//int att[]= {0,0,-40,-40,10,-10,-20,-30};
		int att[]= {-20,0,-40,-40,0,-10,-20,-30};
		buf[0]=0;
		buf[1]=0;
		data.append((char*)buf,2);
		if (txChanged>0)
		txChanged--;

		hpsdrSocket->readDatagram((char*)buf, sizeof(buf), &sender, &senderPort);

		if ((buf[0] == 0xef) && (buf[1] == 0xfe)) {
			switch (buf[2]) {
				case 1: // USB-Packages
					switch (buf[3]) {	// Endpoint
						case 2: 
							for (int frame=0;frame<2;frame++) {
								c = buf+11+512*frame;
								// printf("%02x %02x %02x %02x %02x\n",c[0],c[1],c[2],c[3],c[4]); 
								// 4 byte sequence 
								// 3 Byte sync
								if ((txChanged==0) && ((config[CMD_TX] ^ c[0]) & 1) == 1)
									controlCommand(SRC_CTL, CMD_TX, c[0] & 1, false);
								switch (c[0]>>1) {
									case 0:
										controlCommand(SRC_CTL, CMD_SAMPLE_RATE, rate[c[1]&3], false);
										controlCommand(SRC_CTL, CMD_PREAMP, att[c[3]&7], false);
										hp_receivers = ((c[4]>>3) & 3) + 1;
										break;
									case 1:
										controlCommand(SRC_CTL, CMD_TX_FREQ,  (c[1]<<24) | (c[2]<<16) | (c[3]<<8) | (c[4]), false);
										break;
									case 2:
										controlCommand(SRC_CTL, CMD_RX_FREQ,  (c[1]<<24) | (c[2]<<16) | (c[3]<<8) | (c[4]), false);
										break;
									case 9:
										controlCommand(SRC_CTL, CMD_TX_POWER,  c[1], false);
										break;
								}
								for (int i=16+512*frame;i<8+512+512*frame;i+=8) {
									data.append((char*)buf+7+i,1);
									data.append((char*)buf+6+i,1);
									data.append((char*)buf+5+i,1);
									data.append((char*)buf+4+i,1);
								}
							}
							sdr->writeHiqsdrIQ(data);
							break;
						default:
							;
					}

					break;

				case 2:	// discover
					buf[3] = 0xb8; // MAC0
					buf[4] = 0x88;
					buf[5] = 0xe3;
					buf[6] = 0xed;
					buf[7] = 0x4e;
					buf[8] = 0x81; // MAC6
					buf[9] = METIS_FIRM_REV; // firmware version
					hpsdrSocket->writeDatagram(QByteArray((const char*)buf, 64), sender, senderPort);
					break;

				case 3:	// set ip
					break;

				case 4: // command
					if (buf[3] & 1) {
						// start mic + iq
						 controlCommand(SRC_CTL,  CMD_INTERNAL, C_INTERNAL_HPSDR, false);
					} 
					if (buf[3] & 2) {
						// start bandscope
					}
					if (buf[3] == 0) {
						// stop
						 controlCommand(SRC_CTL,  CMD_INTERNAL, C_INTERNAL_INT, false);
					}
					break;

				default:
					break;
			}
		}
	}
}

void Control::audioRXRaw(QByteArray data) {
	static int seq=0;
	int padding[]={0,0,0,4,10,24,10,20,4};

	if (!config[CMD_INTERNAL]==C_INTERNAL_HPSDR) 
		return;

	for (int i=2;i<data.size();i+=6) {
		if (hpRxBufPos==0) {
			hpRxBuf[hpRxBufPos++]=0xef;
			hpRxBuf[hpRxBufPos++]=0xfe;
			hpRxBuf[hpRxBufPos++]=0x01;
			hpRxBuf[hpRxBufPos++]=0x06; // EP
			hpRxBuf[hpRxBufPos++]=seq>>24;
			hpRxBuf[hpRxBufPos++]=seq>>16;
			hpRxBuf[hpRxBufPos++]=seq>> 8;
			hpRxBuf[hpRxBufPos++]=seq++  ;
		}

		if (hpRxBufPos == 512+8-padding[hp_receivers])
			hpRxBufPos += padding[hp_receivers];

		if (hpRxBufPos==8 || hpRxBufPos==512+8) {
			memset(c,0,sizeof(c));
			config[CMD_TX] ? c[0]=1 : c[0]=0;
			switch (cCnt) {
				case 0:
				default:
					c[2]=MERCURY_FIRM_REV;
					c[3]=PENELOPE_FIRM_REV;
					c[4]=METIS_FIRM_REV;
					cCnt=0;
					break;
			}
			memset(hpRxBuf+hpRxBufPos,0x7f,3);
			hpRxBufPos+=3;
			memcpy(hpRxBuf+hpRxBufPos,c,sizeof(c));
			hpRxBufPos+=5;
		}

		hpRxBuf[hpRxBufPos++]=*(data.data()+i+5);
		hpRxBuf[hpRxBufPos++]=*(data.data()+i+4);
		hpRxBuf[hpRxBufPos++]=*(data.data()+i+3);
		hpRxBuf[hpRxBufPos++]=*(data.data()+i+2);
		hpRxBuf[hpRxBufPos++]=*(data.data()+i+1);
		hpRxBuf[hpRxBufPos++]=*(data.data()+i+0);

		if (hp_receivers>1)
			hpRxBufPos += (hp_receivers-1)*6+2;

		if (hpRxBufPos == 2*512+8-padding[hp_receivers])
			hpRxBufPos += padding[hp_receivers];
		if (hpRxBufPos >= (2*512+8)) {
			hpsdrSocket->writeDatagram(QByteArray((const char*)hpRxBuf, hpRxBufPos), sender, senderPort);
			hpRxBufPos = 0;
			memset(hpRxBuf, 0, sizeof(hpRxBuf));
		}
	}
}
