#include <QUdpSocket>
#include <QTimer>
#include <QThread>
#include <QSettings>
#include <stdint.h>
#include "firdes.h"
#include "dsp.h"

#define RX_EXT			0x80

#define HIQSDR_RX_PORT		0xbc77 
#define HIQSDR_CTL_PORT		0xbc78 
#define HIQSDR_TX_PORT		0xbc79 
#define HIQSDR_RAW_PORT		0xbc7a 
#define HIQSDR_RX_FIR_PORT	0xbc7b 
#define HIQSDR_TX_FIR_PORT	0xbc7c 

#define RASDR_RAW_PORT	22222
#define RASDR_CTL_PORT	22223
#define RASDR_RX_PORT	22224
#define RASDR_TX_PORT	22225


#define TX_OFF      0
#define TX_CW       1
#define TX_SSB      2
#define USE_EXT_IO  4
#define TX_PTT      8

#define HIQ_PREAMP      0x10
#define HIQ_ANT         0x01

#define FILTER_RX	0
#define FILTER_TX	1

#define PACKETSIZE_RX	1442
#define PACKETSIZE_CTL	22	

#define MAX_ENCODERS 	8

class Hiqsdr : public QThread  {
	Q_OBJECT

	public:
		Hiqsdr(QString ip, int rate);
		~Hiqsdr();

		void run();
		void setRXFreq(int f);
		void setTXFreq(int f);
		void setSampleRate(int rate);
		void setAnt(bool);
		void setPresel(int);
		void setAttenuator(int);
		void setTXLevel(int l);
		void setPtt(bool on);
		void setFdx(bool on);
		void setTXRate(int);
		void setCWMode(bool);
		void setFilter(std::vector<gr_complex> coef, int dest=0);
		void setMode(int);
		void setGain(double);
		void setAGCDec(double);
		void setFFT(int time,int size, int src);
		void setVolume(double);
		void setMicGain(double);
		void setToneTest(bool,  double, double, double, double);
		void startRX();
		void stopRX();
		void setActive(bool);
		void setAudioOff(bool);
		void setTxDelay(int);
		void setNBLevel(int);
		void setNotch(int);
		void setSquelch(int);

	public slots:
		void writeHiqsdr(QByteArray);
		void writeHiqsdrIQ(QByteArray);
        void terminate();

	signals:
		void fftData(QByteArray);
		void audioRX(QByteArray);
		void audioRXRaw(QByteArray);
        void controlCommand(int,int,int);

	private slots:
		void readHiqsdr();
		void readCtlHiqsdr();
		void timeout();
		void agcTimeout();
		void muteTimeout();
		void filterTimeout();
		void fftTimeout();
		void readRaw();
		void reReadRaw();
		void setNextFilter(int dest, int reset);

	private:
		int diffStep(unsigned int reg, int val);
		QTimer *muteTimer, *timer, *agcTimer, *filterTimer, *fftTimer;
		QSettings *settings;
		int fftTime;
		int fftSize;
		unsigned char ctlBuf[24];
		unsigned char oldCtlBuf[24];
		unsigned char firBuf[80];
		int firmVersion;
		int byteMode;
		bool cwMode;
		int rawSrc;
		QUdpSocket *ctlSocket;
		QUdpSocket *rxSocket;
		QUdpSocket *txSocket;
		QUdpSocket *rawHiqSocket;
		QString hiqsdrIP;
		int rawAddr;
		void hiqSend();
		QByteArray uBuf,datagram,rawData;
		Dsp *dsp;
		std::vector<gr_complex> coef;
		std::vector<gr_complex> next_coef[2];
		std::vector<gr_complex>::iterator it;
		QList<QByteArray> txBufList;

		short outBuf[PACKETSIZE_RX/sizeof(short)];
		unsigned char d_temp_buff[PACKETSIZE_RX*2];
		unsigned char udpBuf[600*2+2];
		int txPos;
		int mode;
		bool toneTest;
		bool active;
		int hiqsdrClock;
		double smtrOffset;
	
		int nbLevel;
		int rxFreq;
		int txFreq;
		int sampleRate;
		bool ant;
		int presel;
		int att;
		int txLevel;
		int txRate;
		bool ptt;
		bool fdx;
		int rxBand,txBand;
		int txDel;
		int lastTX;
		int taps;
		int coefPadding;
		int cnt;
		bool setFilterReady;
		int preamp;
        bool audioOff,hiqRun;
		int lastStep[MAX_ENCODERS];
		bool initStep[MAX_ENCODERS];
		bool rawRead;
};

